import {
  WebSocketGateway,
  SubscribeMessage,
  MessageBody,
  ConnectedSocket,
  OnGatewayConnection,
  OnGatewayDisconnect,
  WebSocketServer,
} from '@nestjs/websockets';
import { Server, Socket } from 'socket.io';
import { UseGuards } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { ChatService } from './chat.service';
import { UsersService } from '../users/users.service';
import { NotificationService } from '../notification/notification.service';

interface AuthenticatedSocket extends Socket {
  user: {
    id: string;
    email: string;
    username: string;
  };
}

@WebSocketGateway({
  cors: {
    origin: '*',
  },
})
export class ChatGateway implements OnGatewayConnection, OnGatewayDisconnect {
  @WebSocketServer()
  server: Server;

  private connectedUsers = new Map<string, string>(); // userId -> socketId

  constructor(
    private chatService: ChatService,
    private usersService: UsersService,
    private notificationService: NotificationService,
    private jwtService: JwtService,
  ) {}

  async handleConnection(client: Socket) {
    try {
      const token = client.handshake.auth.token || client.handshake.headers.authorization?.split(' ')[1];
      
      if (!token) {
        client.disconnect();
        return;
      }

      const payload = this.jwtService.verify(token);
      const user = await this.usersService.findOne(payload.sub);
      
      if (!user) {
        client.disconnect();
        return;
      }

      (client as AuthenticatedSocket).user = {
        id: user.id,
        email: user.email,
        username: user.username,
      };

      this.connectedUsers.set(user.id, client.id);
      await this.usersService.updateOnlineStatus(user.id, true);

      client.emit('connected', { message: 'Connected successfully', user: client.user });
      
      // Join user to their chat rooms
      const chatRooms = await this.chatService.getUserChatRooms(user.id);
      chatRooms.forEach(room => {
        client.join(room.id);
      });

      // Notify other users that this user is online
      this.server.emit('user_online', { userId: user.id, username: user.username });
      
    } catch (error) {
      client.disconnect();
    }
  }

  async handleDisconnect(client: AuthenticatedSocket) {
    if (client.user) {
      this.connectedUsers.delete(client.user.id);
      await this.usersService.updateOnlineStatus(client.user.id, false);
      
      // Notify other users that this user is offline
      this.server.emit('user_offline', { userId: client.user.id, username: client.user.username });
    }
  }

  @SubscribeMessage('join_room')
  async handleJoinRoom(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const room = await this.chatService.getChatRoom(data.roomId, client.user.id);
      client.join(data.roomId);
      client.emit('joined_room', { roomId: data.roomId, room });
    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('leave_room')
  async handleLeaveRoom(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    client.leave(data.roomId);
    client.emit('left_room', { roomId: data.roomId });
  }

  @SubscribeMessage('send_message')
  async handleMessage(
    @MessageBody() data: { roomId: string; content: string; type?: string; replyToId?: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const message = await this.chatService.createMessage({
        content: data.content,
        type: data.type || 'text',
        senderId: client.user.id,
        chatRoomId: data.roomId,
        replyToId: data.replyToId,
      });

      const populatedMessage = await this.chatService.getMessage(message.id);
      
      // Send message to all users in the room
      this.server.to(data.roomId).emit('new_message', populatedMessage);

      // Handle mentions
      await this.handleMentions(populatedMessage, client.user.id);

      // Send push notifications to offline users
      await this.notificationService.sendNewMessageNotifications(populatedMessage);

    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('edit_message')
  async handleEditMessage(
    @MessageBody() data: { messageId: string; content: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const message = await this.chatService.editMessage(data.messageId, data.content, client.user.id);
      this.server.to(message.chatRoom.id).emit('message_edited', message);
    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('delete_message')
  async handleDeleteMessage(
    @MessageBody() data: { messageId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    try {
      const message = await this.chatService.deleteMessage(data.messageId, client.user.id);
      this.server.to(message.chatRoom.id).emit('message_deleted', { messageId: data.messageId });
    } catch (error) {
      client.emit('error', { message: error.message });
    }
  }

  @SubscribeMessage('typing_start')
  async handleTypingStart(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    client.to(data.roomId).emit('user_typing', {
      userId: client.user.id,
      username: client.user.username,
    });
  }

  @SubscribeMessage('typing_stop')
  async handleTypingStop(
    @MessageBody() data: { roomId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    client.to(data.roomId).emit('user_stopped_typing', {
      userId: client.user.id,
      username: client.user.username,
    });
  }

  @SubscribeMessage('mark_as_read')
  async handleMarkAsRead(
    @MessageBody() data: { roomId: string; messageId: string },
    @ConnectedSocket() client: AuthenticatedSocket,
  ) {
    // In a full implementation, you would track read receipts
    client.to(data.roomId).emit('message_read', {
      messageId: data.messageId,
      userId: client.user.id,
    });
  }

  private async handleMentions(message: any, senderId: string) {
    if (message.mentions && message.mentions.length > 0) {
      for (const mention of message.mentions) {
        const mentionedUserSocket = this.connectedUsers.get(mention.mentionedUser.id);
        if (mentionedUserSocket) {
          this.server.to(mentionedUserSocket).emit('mentioned', {
            message,
            mentionedBy: senderId,
          });
        }

        // Send push notification for mention
        await this.notificationService.sendMentionNotification(message, mention.mentionedUser.id);
      }
    }
  }

  // Method to send message to specific user
  sendToUser(userId: string, event: string, data: any) {
    const socketId = this.connectedUsers.get(userId);
    if (socketId) {
      this.server.to(socketId).emit(event, data);
    }
  }

  // Method to send message to room
  sendToRoom(roomId: string, event: string, data: any) {
    this.server.to(roomId).emit(event, data);
  }
}